<?php
/**
 * Plugged functions for testing the upcoming changes to WP-Members
 *
 * Functions in this file are for the registration and login form changes
 * to be included in the 2.9.0 update
 *
 * version 1.3
 *
 * wpmem_inc_login, wpmem_inc_changepassword & wpmem_inc_resetpassword
 * Redesigned functions to control requests to the wpmem_login_form
 * function.  The new versions use an associative array for defaults
 * that is merged with any filtered arguments using wp_parse_args.
 * These functions now also contain an array for details of each input.
 * CHANGES READY FOR TESTING
 *
 * wpmem_inc_loginfailed & wpmem_inc_regmessage
 * The redesigned message functions contain an array of default values
 * making it easier to customize specific elements without having to 
 * wait to change the message after it has been generated.  Similar to
 * other updates in this version, the defaults can be customized via
 * a filter of arguments merged using wp_parse_args.
 * CHANGES READY FOR TESTING
 *
 * wpmem_inc_registration
 * Newly designed registration form.  Changed the way the form is built, 
 * but tried to keep output changes to a minimum so as to have low impact
 * on existing users custom filters. Several new filters have been added.
 * This functions now assembles the rows into an array that can be filtered.
 * CHANGES READY FOR TESTING
 *
 * wpmem_login_form
 * Same type of changes as registration form, but it's a smaller form,
 * so hopefully less impact. This functions now assembles the rows into
 * an array that can be filtered. 
 * CHANGES READY FOR TESTING
 *
 * wpmem_create_formfield
 * Updates to this function remove the insertion of a linebreak at the
 * end of the created field. Line breaks are now inserted in the login
 * and registration form functions. No other changes.
 * CHANGES READY FOR TESTING
 *
 * wpmem_do_sc_pages & wpmem_securify
 * These functions are no longer included in this file as the required 
 * changes were made in version 2.8.9.  You will need to be using version
 * 2.8.9 to be testing for 2.9.
 *
 * USAGE INSTRUCTIONS FOR TESTING:
 * Save this file to your /wp-content/plugins/ directory as:
 * wp-members-pluggable.php
 * The replaced functions are pluggable, so this will override the 
 * existing functions.  To uninstall and revert to the production
 * version of these functions, simply delete this file or rename it.
 */

 
/**
 * WP-Members 2.9 Testing Functions
 * 
 * This file is part of the WP-Members plugin by Chad Butler
 * You can find out more about this plugin at http://rocketgeek.com
 * Copyright (c) 2006-2013  Chad Butler
 * WP-Members(tm) is a trademark of butlerblog.com
 *
 * @package WordPress
 * @subpackage WP-Members
 * @author Chad Butler 
 * @copyright 2006-2013
 */


if ( ! function_exists( 'wpmem_inc_login' ) ):
/**
 * Login Dialog
 *
 * Loads the login form for user login
 *
 * @since 1.8
 *
 * @uses apply_filters Calls 'wpmem_inc_login_inputs'
 * @uses apply_filters Calls 'wpmem_inc_login_args'
 * @uses apply_filters Calls 'wpmem_restricted_msg' filter to change the restricted post message content
 * @uses wpmem_login_form()
 *
 * @param  string $page
 * @return string $str the generated html for the login form
 */
function wpmem_inc_login( $page="page" )
{ 	
	global $wpmem_regchk;

	$str = '';

	if( $page == "page" ){
	     if( $wpmem_regchk!="success" ){

			$arr = get_option( 'wpmembers_dialogs' );
			
			// this shown above blocked content
			$str = '<p>' . __( stripslashes( $arr[0] ), 'wp-members' ) . '</p>';
			
			// filter blocked content message
			$str = apply_filters( 'wpmem_restricted_msg', $str );

		} 	
	} 
	
	/** create the default inputs **/
	$default_inputs = array(
			array(
				'name'   => __( 'Username', 'wp-members' ), 
				'type'   => 'text', 
				'tag'    => 'log',
				'class'  => 'username',
				'div'    => 'div_text'
			),
			array( 
				'name'   => __( 'Password', 'wp-members' ), 
				'type'   => 'password', 
				'tag'    => 'pwd', 
				'class'  => 'password',
				'div'    => 'div_text'
			)
	);
	
	$default_inputs = apply_filters( 'wpmem_inc_login_inputs', $default_inputs );
	
    $defaults = array( 
		'heading'      => __( 'Existing Users Log In', 'wp-members' ), 
		'action'       => 'login', 
		'button_text'  => __( 'Log In', 'wp-members' ),
		'inputs'       => $default_inputs
	);	
	
	$args = apply_filters( 'wpmem_inc_login_args', '' );

	$arr  = wp_parse_args( $args, $defaults );
	
	$str  = $str . wpmem_login_form( $page, $arr );
	
	return $str;
}
endif;


if ( ! function_exists( 'wpmem_inc_changepassword' ) ):
/**
 * Change Password Dialog
 *
 * Loads the form for changing password.
 *
 * @since 2.0
 *
 * @uses apply_filters Calls 'wpmem_inc_changepassword_inputs'
 * @uses apply_filters Calls 'wpmem_inc_changepassword_args'
 * @uses wpmem_login_form()
 *
 * @return string $str the generated html for the change password form
 */
function wpmem_inc_changepassword()
{
	/** create the default inputs **/
	$default_inputs = array(
			array(
				'name'   => __('New Password', 'wp-members'), 
				'type'   => 'password',
				'tag'    => 'pass1',
				'class'  => 'password',
				'div'    => 'div_text'
			),
			array( 
				'name'   => __('Repeat Password', 'wp-members'), 
				'type'   => 'password', 
				'tag'    => 'pass2',
				'class'  => 'password',
				'div'    => 'div_text'
			)
	);
	
	$default_inputs = apply_filters( 'wpmem_inc_changepassword_inputs', $default_inputs );
	
	$defaults = array(
		'heading'      => __('Change Password', 'wp-members'), 
		'action'       => 'pwdchange', 
		'button_text'  => __('Update Password', 'wp-members'), 
		'inputs'       => $default_inputs
	);

	$args = apply_filters( 'wpmem_inc_changepassword_args', '' );

	$arr  = wp_parse_args( $args, $defaults );

    $str  = wpmem_login_form( 'page', $arr );
	
	return $str;
}
endif;


if ( ! function_exists( 'wpmem_inc_resetpassword' ) ):
/**
 * Reset Password Dialog
 *
 * Loads the form for resetting password.
 *
 * @since 2.1
 *
 * @uses apply_filters Calls 'wpmem_inc_resetpassword_inputs'
 * @uses apply_filters Calls 'wpmem_inc_resetpassword_args'
 * @uses wpmem_login_form()
 *
 * @return string $str the generated html fo the reset password form
 */
function wpmem_inc_resetpassword()
{ 
	/** create the default inputs **/
	$default_inputs = array(
			array(
				'name'   => __('Username', 'wp-members'), 
				'type'   => 'text',
				'tag'    => 'user', 
				'class'  => 'username',
				'div'    => 'div_text'
			),
			array( 
				'name'   => __('Email', 'wp-members'), 
				'type'   => 'text', 
				'tag'    => 'email', 
				'class'  => 'password',
				'div'    => 'div_text'
			)
	);
	
	$default_inputs = apply_filters( 'wpmem_inc_resetpassword_inputs', $default_inputs );
	
	$defaults = array(
		'heading'      => __('Reset Forgotten Password', 'wp-members'), 
		'action'       => 'pwdreset', 
		'button_text'  => __('Reset Password', 'wp-members'), 
		'inputs'       => $default_inputs
	);

	$args = apply_filters( 'wpmem_inc_resetpassword_args', '' );

	$arr  = wp_parse_args( $args, $defaults );

    $str  = wpmem_login_form( 'page', $arr );
	
	return $str;
}
endif;


if ( ! function_exists( 'wpmem_login_form' ) ):
/**
 * Login Form Dialog
 *
 * Builds the form used for login, change password, and reset password.
 *
 * @since 2.5.1
 *
 * @uses apply_filters Calls 'wpmem_login_form_args' filters the default tags and other elements
 * @uses apply_filters Calls 'wpmem_login_form_rows' filters the rows array before the form is built
 * @uses apply_filters Calls 'wpmem_login_hidden_fields' filters the section of hidden fields
 * @uses apply_filters Calls 'wpmem_login_form_buttons' filters the section of buttons
 * @uses apply_filters Calls 'wpmem_forgot_link' filter to change the link for resetting a forgotten password
 * @uses apply_filters Calls 'wpmem_forgot_link_str' NAME MAY CHANGE
 * @uses apply_filters Calls 'wpmem_reg_link' filter to change the link to the registration page
 * @uses apply_filters Calls 'wpmem_reg_link_str' NAME MAY CHANGE
 * @uses apply_filters Calls 'wpmem_login_form' filter to change any element in the generated string of HTML for the login form
 * @uses apply_filters Calls 'wpmem_login_form_before' filter to add any string to the beginning of the login form
 *
 * @param  string $page 
 * @param  array  $arr   The elements needed to generate the form (login|reset password|forgotten password)
 * @return string $form  The HTML for the form as a string
 */
function wpmem_login_form( $page, $arr ) 
{
	// set up default wrappers
	$defaults = array(
		
		// wrappers
		'heading_before'  => '<legend>',
		'heading_after'   => '</legend>',
		'fieldset_before' => '<fieldset>',
		'fieldset_after'  => '</fieldset>',
		'main_div_before' => '<div id="wpmem_login">',
		'main_div_after'  => '</div>',
		'txt_before'      => '[wpmem_txt]',
		'txt_after'       => '[/wpmem_txt]',
		'row_before'      => '',
		'row_after'       => '',
		'buttons_before'  => '<div class="button_div">',
		'buttons_after'   => '</div>',
		'link_before'     => '<div align="right" class="link-text">',
		'link_after'      => '</div>',
		
		// classes & ids
		'form_id'         => '',
		'form_class'      => 'form',
		'button_id'       => '',
		'button_class'    => 'buttons',
		
		// other
		'strip_breaks'    => true,
		'wrap_inputs'     => true,
		'n'               => "\n",
		't'               => "\t",
		
	);
	
	// filter $args
	$args = apply_filters( 'wpmem_login_form_args', '' );
	
	// merge $args with defaults and extract
	extract( wp_parse_args( $args, $defaults ) );

	// extract the array
	extract( $arr );
	
	// build the input rows
	foreach ( $inputs as $input ) {
		$label = '<label for="' . $input['tag'] . '">' . $input['name'] . '</label>';
		$field = wpmem_create_formfield( $input['tag'], $input['type'], '', '', $input['class'] );
		$field_before = ( $wrap_inputs ) ? '<div class="' . $input['div'] . '">' : '';
		$field_after  = ( $wrap_inputs ) ? '</div>' : '';
		$rows[] = array( 
			'row_before'   => $row_before,
			'label'        => $label,
			'field_before' => $field_before,
			'field'        => $field,
			'field_after'  => $field_after,
			'row_after'    => $row_after
		);
	}
	
	// filter the rows array
	$rows = apply_filters( 'wpmem_login_form_rows', $rows, $action );
	
	// put the rows from the array into $form
	$form = '';
	foreach( $rows as $row_item ) {
		$row  = ( $row_item['row_before']   != '' ) ? $row_item['row_before'] . $n . $row_item['label'] . $n : $row_item['label'] . $n;
		$row .= ( $row_item['field_before'] != '' ) ? $row_item['field_before'] . $n . $t . $row_item['field'] . $n . $row_item['field_after'] . $n : $row_item['field'] . $n;
		$row .= ( $row_item['row_before']   != '' ) ? $row_item['row_after'] . $n : '';
		$form.= $row;
	}

	// build hidden fields, filter, and add to the form
	$redirect_to = ( isset( $_REQUEST['redirect_to'] ) ) ? esc_url( $_REQUEST['redirect_to'] ) : get_permalink();
	$hidden = wpmem_create_formfield( 'redirect_to', 'hidden', $redirect_to ) . $n;
	$hidden = $hidden . wpmem_create_formfield( 'a', 'hidden', $action ) . $n;
	$hidden = ( $action != 'login' ) ? $hidden . wpmem_create_formfield( 'formsubmit', 'hidden', '1' ) : $hidden;
	$form   = $form . apply_filters( 'wpmem_login_hidden_fields', $hidden );

	// build the buttons, filter, and add to the form
	if ( $action == 'login' ) {
		$buttons = $t . wpmem_create_formfield( 'rememberme', 'checkbox', 'forever' ) . '&nbsp;' . __('Remember me', 'wp-members') . '&nbsp;&nbsp;'
			. $n . $t . '<input type="submit" name="Submit" value="' . $button_text . '" class="' . $button_class . '" />' . $n;
	} else {
		$buttons = '<input type="submit" name="Submit" value="' . $button_text . '" class="' . $button_class . '" />' . $n;
	}
	
	$form = $form . apply_filters( 'wpmem_login_form_buttons', $buttons_before . $n . $buttons . $buttons_after . $n );

	if ( ( WPMEM_MSURL != null || $page == 'members' ) && $action == 'login' ) { 
		
		$link = apply_filters( 'wpmem_forgot_link', wpmem_chk_qstr( WPMEM_MSURL ) . 'a=pwdreset' );	
		$str  = __( 'Forgot password?', 'wp-members' ) . '&nbsp;<a href="' . $link . '">' . __( 'Click here to reset', 'wp-members' ) . '</a>';
		$form = $form . $link_before . apply_filters( 'wpmem_forgot_link_str', $str ) . $link_after . $n;
		
	}
	
	if ( ( WPMEM_REGURL != null ) && $action == 'login' ) { 

		$link = apply_filters( 'wpmem_reg_link', WPMEM_REGURL );
		$str  = __( 'New User?', 'wp-members' ) . '&nbsp;<a href="' . $link . '">' . __( 'Click here to register', 'wp-members' ) . '</a>';
		$form = $form . $link_before . apply_filters( 'wpmem_reg_link_str', $str ) . $link_after . $n;
		
	}			
	
	// apply the heading
	$form = $heading_before . $heading . $heading_after . $n . $form;
	
	// apply fieldset wrapper
	$form = $fieldset_before . $n . $form . $fieldset_after . $n;
	
	// apply form wrapper
	$form = '<form action="' . get_permalink() . '" method="POST" id="' . $form_id . '" class="' . $form_class . '">' . $n . $form . '</form>';
	
	// apply anchor
	$form = '<a name="login"></a>' . $n . $form;
	
	// apply main wrapper
	$form = $main_div_before . $n . $form . $n . $main_div_after;
	
	// apply wpmem_txt wrapper
	$form = $txt_before . $form . $txt_after;
	
	// remove line breaks
	$form = ( $strip_breaks ) ? str_replace( array( "\n", "\r", "\t" ), array( '','','' ), $form ) : $form;
	
	// apply filters
	$form = apply_filters( 'wpmem_login_form', $form );
	$form = apply_filters( 'wpmem_login_form_before', '' ) . $form;
	
	return $form;
} // end wpmem_login_form
endif;


if ( ! function_exists( 'wpmem_inc_registration' ) ):
/**
 * Registration Form Dialog
 *
 * Outputs the form for new user registration and existing user edits.
 *
 * @since 2.5.1
 *
 * @uses apply_filters Calls 'wpmem_register_form_args' filters array of default tags, labels, and small items
 * @uses apply_filters Calls 'wpmem_register_fields_arr' filters the fields array - THIS MAY CHANGE
 * @uses apply_filters Calls 'wpmem_register_form_rows' filters the form field rows array before adding to the form
 * @uses apply_filters Calls 'wpmem_tos_link_txt' filter to change the text (and link) for the TOS (Terms of Service) acknowledgement
 * @uses apply_filters Calls 'wpmem_register_captcha_row' filters the captcha row before it is added to the form
 * @uses apply_filters Calls 'wpmem_register_hidden_fields' filters the hidden fields before adding to the form
 * @uses apply_filters Calls 'wpmem_register_form_buttons' filters the buttons wrapper and inputs before adding to the form
 * @uses apply_filters Calls 'wpmem_register_heading' filter to change the heading of the registration form
 * @uses apply_filters Calls 'wpmem_register_form_before' filter to add any string to the beginning of the registration form
 * @uses apply_filters Calls 'wpmem_register_form' filter to change any element of the generated HTML string of the registration form
 *
 * @param  string $toggle       (optional) Toggles between new registration ('new') and user profile edit ('edit')
 * @param  string $heading      (optional) The heading text for the form, null (default) for new registration
 * @global string $wpmem_regchk Used to determine if the form is in an error state
 * @global array  $userdata     Used to get the user's registration data if they are logged in (user profile edit)
 * @return string $form         The HTML for the entire form as a string
 */
function wpmem_inc_registration( $toggle = 'new', $heading = '' )
{	
	global $wpmem_regchk, $userdata; 
	
	// set up default wrappers
	$defaults = array(
		
		// wrappers
		'heading_before'   => '<legend>',
		'heading_after'    => '</legend>',
		'fieldset_before'  => '<fieldset>',
		'fieldset_after'   => '</fieldset>',
		'main_div_before'  => '<div id="wpmem_reg">',
		'main_div_after'   => '</div>',
		'txt_before'       => '[wpmem_txt]',
		'txt_after'        => '[/wpmem_txt]',
		'row_before'       => '',
		'row_after'        => '',
		'buttons_before'   => '<div class="button_div">',
		'buttons_after'    => '</div>',
		
		// classes & ids
		'form_id'          => '',
		'form_class'       => 'form',
		'button_id'        => '',
		'button_class'     => 'buttons',
		
		// required field tags and text
		'req_mark'         => '<font class="req">*</font>',
		'req_label'        => __( 'Required field', 'wp-members' ),
		'req_label_before' => '<div class="req-text">',
		'req_label_after'  => '</div>',
		
		// buttons
		'show_clear_form'  => true,
		'clear_form'       => __( 'Reset Form', 'wp-members' ),
		'submit_register'  => __( 'Register', 'wp-members' ),
		'submit_update'    => __( 'Update Profile', 'wp-members' ),
		
		// other
		'strip_breaks'     => true,
		'use_nonce'        => false,
		'wrap_inputs'      => true,
		'n'                => "\n",
		't'                => "\t",
		
	);
	
	// filter $args
	$args = apply_filters( 'wpmem_register_form_args', '', $toggle );
	
	// merge $args with defaults and extract
	extract( wp_parse_args( $args, $defaults ) );
	
	// Username is editable if new reg, otherwise user profile is not
	if( $toggle == 'edit' ) {
		// this is the User Profile edit - username is not editable
		$val   = $userdata->user_login;
		$label = '<label for="username" class="text">' . __( 'Username', 'wp-members' ) . '</label>';
		$input = '<p class="noinput">' . $val . '</p>';
		$field_before = ( $wrap_inputs ) ? '<div class="div_text">' : '';
		$field_after  = ( $wrap_inputs ) ? '</div>' : '';
	} else { 
		// this is a new registration
		$val   = ( isset( $_POST['log'] ) ) ? stripslashes( $_POST['log'] ) : '';
		$label = '<label for="username" class="text">' . __( 'Choose a Username', 'wp-members' ) . $req_mark . '</label>';
		$input = wpmem_create_formfield( 'log', 'text', $val, '', 'username' );

	}
	$field_before = ( $wrap_inputs ) ? '<div class="div_text">' : '';
	$field_after  = ( $wrap_inputs ) ? '</div>': '';
	
	// add the username row to the array
	$rows[] = array( 
		'meta'         => 'username', 
		'type'         => 'text', 
		'val'          => $val,  
		'row_before'   => $row_before,
		'label'        => $label,
		'field_before' => $field_before,
		'field'        => $input,
		'field_after'  => $field_after,
		'row_after'    => $row_after
	);
	
	// loop through the remaining fields
	//$wpmem_fields = get_option( 'wpmembers_fields' );
	$wpmem_fields = apply_filters( 'wpmem_register_fields_arr', get_option( 'wpmembers_fields' ), $toggle );
	
	foreach( $wpmem_fields as $field )
	{
		// start with a clean row
		$val = ''; $label = ''; $input = ''; $field_before = ''; $field_after = '';
		
		// skips user selected passwords for profile update
		$do_row = ( $toggle == 'edit' && $field[2] == 'password' ) ? false : true;
		
		// skips tos, makes tos field hidden on user edit page, unless they haven't got a value for tos
		if( $field[2] == 'tos' && $toggle == 'edit' && ( get_user_meta( $userdata->ID, 'tos', true ) ) ) { 
			$do_row = false; 
			$hidden_tos = wpmem_create_formfield( $field[2], 'hidden', get_user_meta( $userdata->ID, 'tos', true ) );
		}
		
		// if the field is set to display and we aren't skipping, construct the row
		if( $field[4] == 'y' && $do_row == true ) {

			// label for all but TOS
			if( $field[2] != 'tos' ) {

				$class = ( $field[3] == 'password' ) ? 'text' : $field[3];
				
				$label = '<label for="' . $field[2] . '" class="' . $class . '">' . __( $field[1], 'wp-members' );
				$label = ( $field[5] == 'y' ) ? $label . $req_mark : $label;
				$label = $label . '</label>';

			} 

			// gets the field value for both edit profile and submitted reg w/ error
			if( ( $toggle == 'edit' ) && ( $wpmem_regchk != 'updaterr' ) ) { 

				switch( $field[2] ) {
					case( 'description' ):
						$val = htmlspecialchars( get_user_meta( $userdata->ID, 'description', 'true' ) );
						break;

					case( 'user_email' ):
						$val = $userdata->user_email;
						break;

					case( 'user_url' ):
						$val = esc_url( $userdata->user_url );
						break;

					default:
						$val = htmlspecialchars( get_user_meta( $userdata->ID, $field[2], 'true' ) );
						break;
				}

			} else {

				$val = ( isset( $_POST[ $field[2] ] ) ) ? $_POST[ $field[2] ] : '';

			}
			
			// does the tos field
			if( $field[2] == 'tos' ) {

				$val = ( isset( $_POST[ $field[2] ] ) ) ? $_POST[ $field[2] ] : ''; 

				// should be checked by default? and only if form hasn't been submitted
				$val   = ( ! $_POST && $field[8] == 'y' ) ? $field[7] : $val;
				$input = wpmem_create_formfield( $field[2], $field[3], $field[7], $val );
				$input = ( $field[5] == 'y' ) ? $input . $req_mark : $input;

				// determine if TOS is a WP page or not...
				$tos_content = stripslashes( get_option( 'wpmembers_tos' ) );
				//if( has_shortcode( $tos_content, 'wp-members' ) ) {
				if( wpmem_test_shortcode() ) {
					$link = do_shortcode( $tos_content );
					$tos_pop = '<a href="' . $link . '" target="_blank">';
				} else { 
					$tos_pop = "<a href=\"#\" onClick=\"window.open('" . WP_PLUGIN_URL . "/wp-members/wp-members-tos.php','mywindow');\">";
				}
				$input.= apply_filters( 'wpmem_tos_link_txt', sprintf( __( 'Please indicate that you agree to the %s TOS %s', 'wp-members' ), $tos_pop, '</a>' ), $toggle );
				
				// in previous versions, the div class would end up being the same as the row before.
				$field_before = ( $wrap_inputs ) ? '<div class="div_text">' : '';
				$field_after  = ( $wrap_inputs ) ? '</div>' : '';

			} else {

				// for checkboxes
				if( $field[3] == 'checkbox' ) { 
					$valtochk = $val;
					$val = $field[7]; 
					// if it should it be checked by default (& only if form not submitted), then override above...
					if( $field[8] == 'y' && ( ! $_POST && $toggle != 'edit' ) ) { $val = $valtochk = $field[7]; }
				}

				// for dropdown select
				if( $field[3] == 'select' ) {
					$valtochk = $val;
					$val = $field[7];
				}

				if( ! isset( $valtochk ) ) { $valtochk = ''; }
				
				// for all other input types
				$input = wpmem_create_formfield( $field[2], $field[3], $val, $valtochk );
				
				// determine input wrappers
				$field_before = ( $wrap_inputs ) ? '<div class="div_' . $class . '">' : '';
				$field_after  = ( $wrap_inputs ) ? '</div>' : '';
			}

		}

		// if the row is set to display, add the row to the form array
		if( $field[4] == 'y' ) {
			$rows[] = array( 
				'meta'         => $field[2], 
				'type'         => $field[3], 
				'val'          => $val,  
				'row_before'   => $row_before,
				'label'        => $label,
				'field_before' => $field_before,
				'field'        => $input,
				'field_after'  => $field_after,
				'row_after'    => $row_after
			);
		}
	}

	// filter the rows array
	$rows = apply_filters( 'wpmem_register_form_rows', $rows, $toggle );
	
	// put the rows from the array into $form
	$form = '';
	foreach( $rows as $row_item ) {
		$row  = ( $row_item['row_before']   != '' ) ? $row_item['row_before'] . $n . $row_item['label'] . $n : $row_item['label'] . $n;
		$row .= ( $row_item['field_before'] != '' ) ? $row_item['field_before'] . $n . $t . $row_item['field'] . $n . $row_item['field_after'] . $n : $row_item['field'] . $n;
		$row .= ( $row_item['row_before']   != '' ) ? $row_item['row_after'] . $n : '';
		$form.= $row;
	}
	
	// do captcha if enabled
	if( WPMEM_CAPTCHA == 1 && $toggle != 'edit' ) { // don't show on edit page!
		
		// get the captcha options
		$wpmem_captcha = get_option('wpmembers_captcha'); 
		
		// start with a clean row
		$row = '';
		
		if( $wpmem_captcha[0] && $wpmem_captcha[1] ) {
			$row = '<div class="clear"></div>';
			$row.= '<div align="right" class="captcha">' . wpmem_inc_recaptcha( $wpmem_captcha[0], $wpmem_captcha[2] ) . '</div>';
		} 
		
		// add the captcha row to the form
		$form.= apply_filters( 'wpmem_register_captcha_row', $row_before . $row . $row_after, $toggle );
	}

	// create hidden fields
	$var    = ( $toggle == 'edit' ) ? 'update' : 'register';
	$hidden = '<input name="a" type="hidden" value="' . $var . '" />' . $n;
	$hidden.= '<input name="redirect_to" type="hidden" value="' . get_permalink() . '" />' . $n;
	$hidden = ( isset( $hidden_tos ) ) ? $hidden . $hidden_tos . $n : $hidden;
	
	// add the hidden fields to the form
	$form.= apply_filters( 'wpmem_register_hidden_fields', $hidden, $toggle );
	
	// create buttons and wrapper
	$button_text = ( $toggle == 'edit' ) ? $submit_update : $submit_register;
	$buttons = ( $show_clear_form ) ? '<input name="reset" type="reset" value="' . $clear_form . '" class="' . $button_class . '" /> ' . $n : '';
	$buttons.= '<input name="submit" type="submit" value="' . $button_text . '" class="' . $button_class . '" />' . $n;
	
	// add the buttons to the form
	$form.= $buttons_before . $n . apply_filters( 'wpmem_register_form_buttons', $buttons, $toggle ) . $buttons_after . $n;

	// add the required field notation to the bottom of the form
	$form.= $req_label_before . $req_mark . $req_label . $req_label_after;
	
	// apply the heading
	$heading = ( !$heading ) ? apply_filters( 'wpmem_register_heading', __( 'New User Registration', 'wp-members' ), $toggle ) : $heading;
	$form = $heading_before . $heading . $heading_after . $n . $form;
	
	// apply fieldset wrapper
	$form = $fieldset_before . $n . $form . $n . $fieldset_after;
	
	// apply attribution if enabled
	$form = $form . wpmem_inc_attribution();
	
	// apply nonce
	$form = ( defined( 'WPMEM_USE_NONCE' ) || $use_nonce ) ? wp_nonce_field( 'wpmem-validate-submit', 'wpmem-form-submit' ) . $n . $form : $form;
	
	// apply form wrapper
	$form = '<form name="form" method="post" action="' . get_permalink() . '" id="' . $form_id . '" class="' . $form_class . '">' . $n . $form. $n . '</form>';
	
	// apply anchor
	$form = '<a name="register"></a>' . $n . $form;
	
	// apply main div wrapper
	$form = $main_div_before . $n . $form . $n . $main_div_after . $n;
	
	// apply wpmem_txt wrapper
	$form = $txt_before . $form . $txt_after;
	
	// remove line breaks if enabled for easier filtering later
	$form = ( $strip_breaks ) ? str_replace( array( "\n", "\r", "\t" ), array( '','','' ), $form ) : $form;
	
	// apply filters
	$form = apply_filters( 'wpmem_register_form', $form, $toggle );
	$form = apply_filters( 'wpmem_register_form_before', '', $toggle ) . $form;

	// return the generated form
	return $form;
} // end wpmem_inc_registration
endif;


function wpmem_create_formfield( $name, $type, $value, $valtochk=null, $class='textbox' )
{
	switch( $type ) {

	case "checkbox":
		if( $class = 'textbox' ) { $class = "checkbox"; }
		$str = "<input name=\"$name\" type=\"$type\" id=\"$name\" value=\"$value\" " . wpmem_selected( $value, $valtochk, $type ) . " />";
		break;

	case "text":
		$value = stripslashes( $value );
		$str = "<input name=\"$name\" type=\"$type\" id=\"$name\" value=\"$value\" class=\"$class\" />";
		break;

	case "textarea":
		$value = stripslashes( $value );
		if( $class = 'textbox' ) { $class = "textarea"; }
		$str = "<textarea cols=\"20\" rows=\"5\" name=\"$name\" id=\"$name\" class=\"$class\">$value</textarea>";
		break;

	case "password":
		$str = "<input name=\"$name\" type=\"$type\" id=\"$name\" class=\"$class\" />";
		break;

	case "hidden":
		$str = "<input name=\"$name\" type=\"$type\" value=\"$value\" />";
		break;

	case "option":
		$str = "<option value=\"$value\" " . wpmem_selected( $value, $valtochk, 'select' ) . " >$name</option>";
		break;

	case "select":
		if( $class == 'textbox' ) { $class = "dropdown"; }
		$str = "<select name=\"$name\" id=\"$name\" class=\"$class\">\n";
		foreach( $value as $option ) {
			$pieces = explode( '|', $option );
			$str = $str . "<option value=\"$pieces[1]\"" . wpmem_selected( $pieces[1], $valtochk, 'select' ) . ">$pieces[0]</option>\n";
		}
		$str = $str . "</select>";
		break;

	}
	
	return $str;
}


a


if ( ! function_exists( 'wpmem_inc_regmessage' ) ):
/**
 * Message Dialog
 *
 * Returns various dialogs and error messages.
 *
 * @since 1.8
 *
 * @uses apply_filters Calls 'wpmem_msg_defaults' filter to filter the default tags
 * @uses apply_filters Calls 'wpmem_msg_dialog' filter to filter the message dialog
 *
 * @param  string $toggle error message toggle to look for specific error messages
 * @param  string $msg a message that has no toggle that is passed directly to the function
 * @return string $str The final HTML for the message
 */
function wpmem_inc_regmessage( $toggle, $msg = '' )
{
	// defaults
	$defaults = array(
		'div_before' => '<div class="wpmem_msg" align="center">',
		'div_after'  => '</div>', 
		'p_before'   => '<p>',
		'p_after'    => '</p>',
		'toggles'    => array( 
							'user', 
							'email', 
							'success', 
							'editsuccess', 
							'pwdchangerr', 
							'pwdchangesuccess', 
							'pwdreseterr', 
							'pwdresetsuccess' 
						)
	);
	
	// filter $args
	$args = apply_filters( 'wpmem_msg_args', '' );
	
	// merge $args with defaults and extract
	extract( wp_parse_args( $args, $defaults ) );

	// get dialogs set in the db
	$dialogs = get_option( 'wpmembers_dialogs' );

	for( $r = 0; $r < count( $toggles ); $r++ ) {
		if( $toggle == $toggles[$r] ) {
			$msg = stripslashes( $dialogs[$r+1] );
			break;
		}
	}

	$str = $div_before . $p_before . stripslashes( $msg ) . $p_after . $div_after;

	return apply_filters( 'wpmem_msg_dialog', $str );

}
endif;

/** End of File **/